% Detects the shots based on global histogram affinity of consecutive
% frames.
% shots is a Nx2 matrix storing the start and end frames of each shot.
% shot(i,2) + 1 = shots(i+1, 1)
function shots = DetectShots(video)

    min_affinity = 0.7;
    min_relative_change = 0.7;
    affinity_dil = imdilate(video.affinity_frames, ones(5,1));
    affinity_er = imerode(video.affinity_frames, ones(5,1));
    min_locs = find(video.affinity_frames < min_affinity & video.affinity_frames == affinity_er & ...
        affinity_er < min_relative_change .* affinity_dil);
    
    num_frames = numel(video.affinity_frames) + 1;
    num_shots = numel(min_locs) + 1;
    shots = zeros(num_shots, 2);
    shots(1,1) = 1;
    shots(end,2) = num_frames;
    shots(1:end-1, 2) = min_locs;
    shots(2:end, 1) = shots(1:end-1, 2) + 1;
    
    