% Finds the connected components and sorts them by decreasing area
function Xs = FindConnectedComponents(video, Xs)

    M = video.height;
    N = video.width;
%     r = video.adjmat_and_affinity(:,1);
%     c = video.adjmat_and_affinity(:,2);
    r = [video.spatial_neighbours(:,1); video.temporal_neighbours(:,1)];
    c = [video.spatial_neighbours(:,2); video.temporal_neighbours(:,2)];
    num_solutions = size(Xs, 2);
    area = [];
    good_comps = cell(num_solutions, 1);
    min_frames = 5;
    min_area = M*N*0.01;
    
    sup_frames = zeros(size(Xs,1), 1);
    num_frames = size(video.grayscale_frames, 3);
    
    for i = 1:num_frames
        sup_frames(video.sup_idx_range(i,1):video.sup_idx_range(i,2)) = i;
    end
    
    for i = 1:num_solutions
        X = Xs(:,i);
        idx = X(r) & X(c);
        num_sups = numel(X);

        adjmat = sparse(r(idx), c(idx), ones(sum(idx), 1), num_sups, num_sups);
        adjmat = adjmat + adjmat';
        [ci{i}, sizes{i}] = components(adjmat);
        
        % select components with at least 2 superpixels
        big_comps = find(sizes{i} > 1);
        num_big_comps = numel(big_comps);
    
        for comp = 1:num_big_comps
            X_new = (ci{i}==big_comps(comp));

            % select components that are above certain number of frames
            unique_frames = unique(sup_frames(X_new));
            if (numel(unique_frames) < min_frames)
                continue;
            end
            
            % select components that are above a certain area
            cur_area = sum(video.sup_area(X_new));
            if (cur_area < min_area)
                continue;
            end
            
            good_comps{i} = [good_comps{i}; big_comps(comp)];
            area = [area; cur_area];
        end
        
        disp(['Solution #', num2str(i),', connected components so far: ', num2str(numel(area))]);
    end
    
    Xs_new = zeros(size(Xs,1), numel(area));
    k = 1;
    for i = 1:num_solutions
        num_big_comps = numel(good_comps{i});
        for comp = 1:num_big_comps
            Xs_new(:, k) = (ci{i}==good_comps{i}(comp));
            k = k + 1;
        end
    end
    
    [s, idx] = sort(area, 'descend');
    Xs = Xs_new(:, idx);
    