function [affinity, video_out] = GrayscaleFramesAffinity(video, display_on)

    num_frames = size(video.grayscale_frames, 3);
    if (nargin < 2 || isempty(display_on))
        display_on = false;
    else
        video_out.width = video.width;
        video_out.height = video.height;
        video_out.times = video.times;
        video_out.frames = repmat(struct('cdata', [], 'colormap', []), num_frames-1, 1);
    end        
    
    images = video.grayscale_frames;
    affinity = zeros(num_frames-1, 1);
    num_bins = 20;
    color_hist = zeros(num_frames, num_bins);
    for i = 1:num_frames
%         disp(['Processing frame ', num2str(i),' out of ', num2str(num_frames)]);
        img = images(:,:,i);
        
        color_hist(i, :) = histc(img(:), linspace(0, 1, num_bins));
        color_hist(i, :) = color_hist(i, :) / sum(color_hist(i, :));
    end
    
    for i = 1:num_frames-1
        affinity(i) = sum(min(color_hist(i, :), color_hist(i+1, :)));
    end
    
    if (display_on)
        for i = 1:num_frames-1
            subplot(2,1,1);
            if (isfield(video, 'frames') && ~isempty(video.frames))
                imshow(video.frames(i).cdata);
            else
                imshow(images(:,:,i));
            end
            drawnow;
            
            subplot(2,1,2);
            plot(affinity, '-b', 'LineWidth', 2);
            hold on;
            scatter(i, affinity(i), 100, 'r', 'filled');
            hold off;
            xlabel('Frame #', 'Fontsize', 15);
            ylabel('Affinity to the next frame', 'Fontsize', 15);
            set(gca, 'Fontsize', 15);
            
            f = getframe(gcf);
            video_out.frames(i) = f;
        end
    end