% Precompute superpixel descriptors (color histograms, etc...)

function video = PreprocessVideo(video)
    num_sups = sum(max(max(video.superpixels, [], 1), [], 2));
    num_sup_images = size(video.superpixels, 3);
    
    if (isfield(video, 'frames') && ~isempty(video.frames))
        num_bins = 4^3;
    else
        num_bins = 30;
    end
    gray_range = linspace(0,1,num_bins);
    grayscale_hist = zeros(num_sups, num_bins);
    sift_bins = 8;
    sift = zeros(num_sups, sift_bins);
    angle_bins = linspace(-pi, pi, sift_bins+1);
    angle_bins(end) = angle_bins(end + eps);
    
    disp('Computing superpixel histograms');
    num_sup_idx = 1;
    for i = 1:num_sup_images
        disp(['Processing frame ', num2str(i),' out of ', num2str(num_sup_images)]);
        
        num_sups_cur = max(max(video.superpixels(:,:,i), [], 1), [], 2);
        [gx,gy] = height_function_der(video.grayscale_frames(:,:,i));
        grad_mag = abs(complex(gx,gy));
        grad_orient = angle(complex(gx,gy));

        if (isfield(video, 'frames') && ~isempty(video.frames))
            img = video.frames(i).cdata;
        end
        
        for s = 1:num_sups_cur
            pix_idx = (video.superpixels(:,:,i) == s);
            num_pix = sum(pix_idx(:));
            if (isfield(video, 'frames') && ~isempty(video.frames))
                bins = linspace(0, 256, round(num_bins^(1/3))+1); 
                grayscale_hist(s+num_sup_idx-1, :) = reshape(histcn(reshape(...
                    img(repmat(pix_idx, [1,1,3])), [num_pix, 3]), ...
                    bins, bins, bins), [1, num_bins]);
            else
                [r,c] = find(pix_idx);
                grayscale_hist(s+num_sup_idx-1, :) = ...
                    histc(video.grayscale_frames(sub2ind(size(video.grayscale_frames), r, c, i*ones(size(r)))), gray_range);
            end
            
            h = whistc(grad_orient(pix_idx), grad_mag(pix_idx), angle_bins);
            h = h(1:end-1);
            h = h ./ sum(h);
            sift(s+num_sup_idx-1, :) = h;
        end
        num_sup_idx = num_sup_idx + num_sups_cur;
    end
    grayscale_hist = grayscale_hist ./ (repmat(sum(grayscale_hist, 2), [1, num_bins]) + eps);
    video.grayscale_hist = grayscale_hist;
    video.sift = sift;
    
    disp('Computing whole frame affinities');
    video.affinity_frames = GrayscaleFramesAffinity(video);
