function Xs = SpatiotemporalClosure(video, video_prefix)

    changes_made = false;

    if (~isfield(video, 'grayscale_frames'))
        disp('---------- Converting into grayscale --------------');
        video = VideoToGrayscaleFrames(video);
        changes_made = true;
    end
    
    if (changes_made || ~isfield(video, 'Pb'))
        disp('---------- Computing Pb --------------');
        video = VideoPb(video);
        changes_made = true;
    end

    if (changes_made || ~isfield(video, 'Pb_processed'))
        disp('---------- Postprocessing Pb --------------');
        video = PreprocessVideoPb(video);
        changes_made = true;
    end


    if (changes_made || ~isfield(video, 'superpixels'))
        disp('---------- Computing superpixels --------------');
        num_superpixels = video.height * video.width / 200;
        video = SegmentVideoIntoSuperpixels(video, num_superpixels, video_prefix);
        video.sup_type = 'turbopixels';
        changes_made = true;
    end

    if (changes_made || ~isfield(video, 'grayscale_hist'))
        disp('---------- Computing superpixel data --------------');
        video = PreprocessVideo(video);
        changes_made = true;
    end
    
    if (changes_made || ~isfield(video, 'spatial_neigh') || ~isfield(video, 'temporal_neigh'))
        disp('---------- Computing superpixel graph --------------');
        video = BuildSuperpixelGraphVideo(video, 0, 0);
        changes_made = true;
    end
    
    if (changes_made || ~isfield(video, 'adjmat_and_affinity'))
        disp('---------- Computing superpixel affinities --------------');
        video = SuperpixelAffinitiesVideo(video, 1, 6);
    end
    
    Xs = SuperpixelGrouping3D_sparse(video);
    Xs = DetectAreaChange(video, Xs);
    Xs = FindConnectedComponents(video, Xs);
    Xs = RemoveDuplicateSolutions(Xs);

