function video = SuperpixelAffinitiesVideo(video, affinity_type, affinity_exp)

    if (nargin < 2 || isempty(affinity_type))
        affinity_type = 1;
    end
    
    if (nargin < 3 || isempty(affinity_exp))
        affinity_exp = 6;
    end
    
    num_frames = size(video.grayscale_frames, 3);
    num_sups = video.sup_idx_range(end,2);
    
    sup1 = video.adjmat_and_affinity(:,1);
    sup2 = video.adjmat_and_affinity(:,2);
%     sq_dist = video.adjmat_and_affinity(:,3);
%     dist_affinity = exp(-sq_dist/1000);
%     d = sparse(sup1,sup2,dist_affinity,num_sups,num_sups);
%     d = d + d';
%     max_sup_dist_affinity = full(max(d, [], 2));
%     dist_affinity = dist_affinity ./ (max(max_sup_dist_affinity(sup1), max_sup_dist_affinity(sup2)) + eps);
    
%     affinity = video.adjmat_and_affinity(:,3);    
%     affinity = affinity .* sum(min(video.grayscale_hist(sup1, :), video.grayscale_hist(sup2, :)), 2);
    affinity = sum(min(video.grayscale_hist(sup1, :), video.grayscale_hist(sup2, :)), 2);
%     affinity = 1;
    
    motion_affinity = ones(size(sup1));
    
    sup_frames = zeros(num_sups, 1);

    for i = 1:num_frames
        sup_frames(video.sup_idx_range(i,1):video.sup_idx_range(i,2)) = i;
    end

    is_last_frame = double(sup_frames(sup1) == num_frames | sup_frames(sup2) == num_frames);
            
    same_frame = sup_frames(sup1) == sup_frames(sup2);
        
    flow_mag1 = sqrt(sum(video.supSpeed(sup1, :).^2, 2));
    flow_mag2 = sqrt(sum(video.supSpeed(sup2, :).^2, 2));

    flow_diff_mag = sqrt(sum((video.supSpeed(sup1, :) - video.supSpeed(sup2, :)).^2, 2));
    max_flow_mag = max(flow_mag1, flow_mag2);
    flow_dist = flow_diff_mag ./ (max_flow_mag + eps);
    motion_affinity_sup_flow = max(0, min(1, 1-flow_dist));
    
    if (any(isnan(motion_affinity_sup_flow)))
        error('NAN alert');
    end
    
%     motion_affinity_sup_flow = sum(min(video.sup_flow_hist(sup1, :), video.sup_flow_hist(sup2, :)), 2);
    motion_affinity_sup_flow = (~video.supSpeedReliab(sup1) | ~video.supSpeedReliab(sup2)) + ...
        (video.supSpeedReliab(sup1) & video.supSpeedReliab(sup2)) .* ...
        motion_affinity_sup_flow;
%     motion_affinity_motion_profile = MotionProfileAffinity(video.motion_profile(sup1, :), video.motion_profile(sup2, :));

    switch (affinity_type)
        case 0
            motion_affinity = ones(size(sup1));
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 1
            motion_affinity(same_frame) = motion_affinity_sup_flow(same_frame);
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 2
            motion_affinity(same_frame) = motion_affinity_motion_profile(same_frame);
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 3
            motion_affinity(same_frame) = motion_affinity_motion_profile(same_frame) .* motion_affinity_sup_flow(same_frame);
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 4
            motion_affinity = motion_affinity_sup_flow;
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 4.5
            motion_affinity = motion_affinity_sup_flow;
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
            affinity = 1;
        case 5
            motion_affinity = motion_affinity_motion_profile;
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 5.5
            motion_affinity = motion_affinity_motion_profile;
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
            affinity = 1;
        case 6
            motion_affinity = motion_affinity_motion_profile .* motion_affinity_sup_flow;
            area_multiplier = (video.sup_area(sup1).*video.sup_area(sup2));
        case 7
            motion_affinity(same_frame) = motion_affinity_sup_flow(same_frame);
            area_multiplier = sqrt(video.sup_area(sup1).*video.sup_area(sup2));            
    end

    motion_affinity = is_last_frame + (1-is_last_frame) .* motion_affinity;
    affinity = max(0.01, min(1-0.01, min(motion_affinity,affinity)));
%     affinity = max(0.01, min(1-0.01, motion_affinity .* affinity));
    affinity = max(0.01, min(1-0.01, affinity));
    
%     affinity = affinity .* exp(-sq_dist/100);
    
    video.adjmat_and_affinity(:,3) = area_multiplier.*affinity.^affinity_exp;
%     video.adjmat_and_affinity(:,3) = video.sup_area(sup1).*video.sup_area(sup2).*affinity.^5;
%     video.adjmat_and_affinity(:,3) = (video.sup_area(sup1)+video.sup_area(sup2)).*affinity.^5;

