% boundary_constraints is a 4x1 binary vector constraining solutions not to
% be attached to a particular side of the image(bottom, top, left, right)
% For example boundary_costraints = [1;0;1;0] will not find solution that
% touch the bottom or the left side of the image
function [Xs, Gi, Gij, A] = SuperpixelGrouping3D_sparse(video, frame_range, boundary_constraints)
         
    INFINITY = 1e9;

    if (nargin < 3 || isempty(boundary_constraints))
        boundary_constraints = true(4,1);
    end
    
    num_sups = video.sup_idx_range(end,2);
    sup_images = video.superpixels;
%     affinity_sparse = [video.adjmat_and_affinity; video.motion_adjmat_and_affinity];
    affinity_sparse = video.adjmat_and_affinity;
    
    
    boundary_sups = false(num_sups, 1);
    num_sup_idx = 0;
    num_sup_images = size(sup_images, 3);
    A = video.sup_area;
    for i = 1:num_sup_images
        num_sups_cur = max(max(sup_images(:,:,i), [], 1), [], 2);
        
        if (boundary_constraints(1) > 0)
            bot = unique(sup_images(1,sup_images(1,:,i) > 0, i));
            boundary_sups(bot + num_sup_idx) = true;
        end
        
        if (boundary_constraints(2) > 0)
            top = unique(sup_images(end,sup_images(end,:,i) > 0, i));
            boundary_sups(top + num_sup_idx) = true;
        end
        
        if (boundary_constraints(3) > 0)
            left = unique(sup_images(sup_images(:,1,i) > 0, 1, i));
            boundary_sups(left + num_sup_idx) = true;
        end
        
        if (boundary_constraints(4) > 0)
            right = unique(sup_images(sup_images(:,end,i) > 0, end, i));
            boundary_sups(right + num_sup_idx) = true;
        end
        
        num_sup_idx = num_sup_idx + num_sups_cur;
    end
    
    r = [affinity_sparse(:,1); affinity_sparse(:,2)];
    c = [affinity_sparse(:,2); affinity_sparse(:,1)];
    affinity = [affinity_sparse(:,3); affinity_sparse(:,3)];
    affinity = sparse(r, c, affinity, num_sups, num_sups);

    if (exist('frame_range', 'var') && ~isempty(frame_range))
    
        sup_frames = zeros(num_sups, 1);

        for i = 1:num_sup_images
            sup_frames(video.sup_idx_range(i,1):video.sup_idx_range(i,2)) = i;
        end

        selected_sups = find(ismember(sup_frames, frame_range));
        
        A = A(selected_sups);
        affinity = affinity(selected_sups, selected_sups);
        boundary_sups = boundary_sups(selected_sups);
        selected_edges = ismember(affinity_sparse(:,1), selected_sups) & ismember(affinity_sparse(:,2), selected_sups);
        affinity_sparse = affinity_sparse(selected_edges, :);
        affinity_sparse(:,1:2) = affinity_sparse(:,1:2) - min(selected_sups) + 1;
    end
    
%     affinity(boundary_sups, :) = 0;
%     affinity(:, boundary_sups) = 0;
%     [a, b, c] = find(tril(affinity, -1));
%     affinity_sparse = [a,b,c];
%     [affinity_sparse(:,1), affinity_sparse(:,2), affinity_sparse(:,3)] = find(tril(affinity, -1));
    
    Gi = sum(affinity, 2);
%     Gi(boundary_sups) = Gi(boundary_sups) + INFINITY;
    
%     Gi(video.sup_idx_range(1,1):video.sup_idx_range(1,2)) = Gi(video.sup_idx_range(1,1):video.sup_idx_range(1,2))+INFINITY;
%     Gi(video.sup_idx_range(end,1):video.sup_idx_range(end,2)) = Gi(video.sup_idx_range(end,1):video.sup_idx_range(end,2))+INFINITY;
    Gi = full(Gi);
    
    Gij = zeros(size(affinity_sparse));
    Gij(:,1:2) = affinity_sparse(:,1:2) - 1;
    Gij(:,3) = -2*affinity_sparse(:,3);

    
    Xs = ParametricMaxflow_MEX([Gi';Gi' + boundary_sups'.*INFINITY], Gij', -1, 0, 500);
    
    if (exist('frame_range', 'var') && ~isempty(frame_range))
        Xs_selected = false(num_sups, size(Xs,2));
        Xs_selected(selected_sups, :) = Xs;
        Xs = Xs_selected;
    end
    
