function video = SegmentVideoIntoSuperpixels(video, num_sup_from_user, video_prefix, start_frame)

    if (~exist('num_sup_from_user', 'var') || isempty(num_sup_from_user))
        num_sup_from_user = 400;
    end
    
    num_frames = size(video.grayscale_frames, 3);
    
    if (nargout == 1)
        video.superpixels = zeros(size(video.grayscale_frames));
        video.Vx = zeros(size(video.grayscale_frames));
        video.Vy = zeros(size(video.grayscale_frames));
        video.flow_reliab = false(size(video.grayscale_frames));
    end
    
    video.sup_idx_range = [];
    video.sup_centers = [];
    video.sup_area = [];
    video.supSpeed = [];
    video.supSpeedReliab = [];
    
    % Sigma for weighting the flow of pixels in a superpixel
    sigma_motion = (video.width * video.height / num_sup_from_user);
    
    s_global = 1;
    use_pb = true;
        
    if (nargin < 4 || isempty(start_frame))
        start_frame = 1;
    end
      
	if (nargin > 2)
		frame_filename = [video_prefix, num2str(1), '.mat'];
		sup_extra_info_filename = [video_prefix, '_sup_extra_info.mat'];
		if (exist(frame_filename, 'file'))
			load(sup_extra_info_filename);
			video.sup_idx_range = sup_idx_range;
			video.sup_centers = sup_centers;
			video.sup_area = sup_area;
			video.supSpeed = supSpeed;
			video.supSpeedReliab = supSpeedReliab;
		end
	end
	 
    for i = start_frame:num_frames
        disp(['Processing frame ', num2str(i),' out of ', num2str(num_frames)]);
        img = video.grayscale_frames(:,:,i);

		compute_superpixels = true;
		if (nargin > 2)
            frame_filename = [video_prefix, num2str(i), '.mat'];
            if (exist(frame_filename, 'file'))
				load(frame_filename);
				compute_superpixels = false;
			end
		end
		
		if (compute_superpixels)
			if (i > 1)
				seeds = video.sup_centers(video.sup_idx_range(i-1,1):video.sup_idx_range(i-1,2), :);
				good_sups = repmat(video.supSpeedReliab(video.sup_idx_range(i-1,1):video.sup_idx_range(i-1,2)), [1,2]) > 0;
				flow = video.supSpeed(video.sup_idx_range(i-1,1):video.sup_idx_range(i-1,2), :);
				seeds(good_sups) = seeds(good_sups) + flow(good_sups);
				M = video.height;
				N = video.width;
				seeds(:, 1) = max(min(round(seeds(:,1)), N), 1);
				seeds(:, 2) = max(min(round(seeds(:,2)), M), 1);
			else
				seeds = [];
			end

			if (~use_pb)
				[phi, boundary] = superpixels(img, num_sup_from_user, [], [], seeds);
			else
				[phi, boundary] = superpixels_pb(img, num_sup_from_user, [], [], seeds, video.Pb_processed(:,:,i));
			end
			superPixels = bwlabel(~boundary, 4);
            [adjmat, boundary_pixels] = TurbopixelsAdjacency(superPixels);
			num_sups = max(superPixels(:));
			stats = regionprops(superPixels, 'Centroid', 'Area');
			video.sup_idx_range(i, 1) = s_global;
			video.sup_idx_range(i, 2) = s_global + num_sups - 1;
			video.sup_centers = [video.sup_centers; reshape([stats(:).Centroid], [2, numel(stats)])'];
			video.sup_area = [video.sup_area; [stats(:).Area]'];
		        
			if (nargin > 2)
				frame_filename = [video_prefix, num2str(i), '.mat'];
				save(frame_filename, 'superPixels', 'adjmat', 'boundary_pixels');
			end

			if (i < num_frames)
				[Vx,Vy,reliab] = optFlowLk(video.grayscale_frames(:,:,i),  video.grayscale_frames(:,:,i+1), [], 4, 3, 3e-6, 0 );
				reliab = reliab > 3e-6;
				
				if (nargout == 1)
					video.Vx(:,:,i) = Vx;
					video.Vy(:,:,i) = Vy;
					video.flow_reliab(:,:,i) = reliab;
				end
				
				for s = 1:num_sups
					sup_idx = superPixels == s;
					reliable_pixels = reliab(sup_idx);
					numReliab = sum(reliable_pixels);

					[y,x] = find(sup_idx);
					weights = exp(-((x - video.sup_centers(s_global, 1)).^2 + ...
						(y - video.sup_centers(s_global, 2)).^2) / sigma_motion);
					sum_weights = sum(weights(reliable_pixels));

					if (numReliab > 0)
						video.supSpeed(s_global, 1) = sum(Vx(sup_idx & reliab).*weights(reliable_pixels)) / sum_weights;
						video.supSpeed(s_global, 2) = sum(Vy(sup_idx & reliab).*weights(reliable_pixels)) / sum_weights;
						video.supSpeedReliab(s_global,1) = (numReliab / video.sup_area(s_global)) > 0.1;
					else
						video.supSpeed(s_global, 1) = 0;
						video.supSpeed(s_global, 2) = 0;
						video.supSpeedReliab(s_global,1) = false;
					end

					s_global = s_global + 1;
				end
			else
				video.supSpeed = [video.supSpeed; zeros(num_sups,2)];
				video.supSpeedReliab = [video.supSpeedReliab; zeros(num_sups,1)];
			end
			
			save(sup_extra_info_filename, '-struct', 'video', 'sup_idx_range', 'sup_centers', 'sup_area', ...
				'supSpeed', 'supSpeedReliab');
		end
		
		if (nargout == 1)
			video.superpixels(:,:,i) = superPixels;
            video.adjmat{i} = adjmat;
            video.boundary_pixels{i} = boundary_pixels;            
		end
    end

    video.supSpeedReliab = video.supSpeedReliab > 0;
