% Given a superpixel segmentation from the Turbopixels algorithm, where
% there is a 1 pixel wide boundary between superpixels, get the adjacency
% matrix and a square cell array of boundary pixels storing the pixels on every
% superpixel edge
function [adjmat, boundary_pixels] = TurbopixelsAdjacency(sup_image)

    boundaries = sup_image == 0;
    [r,c] = find(boundaries);
    [M,N] = size(boundaries);
    num_sups = max(sup_image(:));
    adjmat = zeros(num_sups, num_sups);
    boundary_pixels = cell(num_sups, num_sups);
    
    for i = 1:numel(r)
        r_range = r(i) + (-1:1);
        c_range = c(i) + (-1:1);
        r_range = r_range(r_range > 0);
        r_range = r_range(r_range <= M);
        c_range = c_range(c_range > 0);
        c_range = c_range(c_range <= N);
        sups = sup_image(r_range, c_range);
        sups = sups(sups > 0);
        sup1 = min(sups);
        sup2 = max(sups);
        
        adjmat(sup1, sup2) = 1;
        adjmat(sup2, sup1) = 1;
        
        try
            boundary_pixels{sup1, sup2} = [boundary_pixels{sup1, sup2}, sub2ind([M,N], r(i), c(i))];
        catch
            xxx = 1;
        end
        boundary_pixels{sup2, sup1} = boundary_pixels{sup1, sup2};
    end
    
%     adjmat(eye(size(adjmat)) == 1) = 0;